import os
import json

from bigcommerce_repricing import logger


class AliexpressInventoryManager(object):
    def __init__(self, bigcommerce_api):
        self.api = bigcommerce_api

    def enumerate_inventory(self, inventory_path):
        if not os.path.isfile(inventory_path):
            self.download_inventory(inventory_path)

        if not os.path.isfile(inventory_path):
            return

        with open(inventory_path) as inv_fh:
            next(inv_fh)
            for line in inv_fh:
                product_id, ali_product_id, variants_str = line.strip().split('\t')
                yield {
                    'product_id': product_id,
                    'ali_product_id': ali_product_id,
                    'variants': json.loads(variants_str)
                }

        yield None

    def download_inventory(self, inventory_path):
        logger.debug('Downloading bigcommerce inventory...')

        products = dict()
        for variant in self.api.CatalogVariants.iterall():
            record = {
                'variant_id': variant.id,
                'sku': variant.sku,
                'price': variant.price,
                'quantity': variant.inventory_level
            }
            if variant.product_id in products:
                products[variant.product_id].append(record)
            else:
                products[variant.product_id] = [record]

        if os.path.isfile(inventory_path):
            os.remove(inventory_path)

        headers = ['product_id', 'ali_product_id', 'variants']
        with open(inventory_path, 'a') as inv_fh:
            inv_fh.write('\t'.join(headers))
            inv_fh.write('\n')

            for product_id, variants in products.items():
                if not variants:
                    logger.warning('[ProductWithNoVariants] product_id: %s', product_id)
                    continue

                ali_product_ids = [variant['sku'].replace('-', '_').split('_').pop(0) \
                    for variant in variants]
                if len(set(ali_product_ids)) > 1:
                    logger.warning('[ProductVariantsWithDifferentAliexpressId] %s', repr(variants))
                    continue

                ali_product_id = ali_product_ids[0]
                inv_fh.write(
                    '\t'.join([str(product_id), str(ali_product_id), json.dumps(variants)]))
                inv_fh.write('\n')

        logger.debug('Bigcommerce inventory has been downloaded.')
