# -*- coding: utf-8 -*-

# Copyright © 2018 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

from website_utils.misc import decrypt
from bigcommerce_repricing import logger


class BigcommerceInventoryManager(object):
    def __init__(self, repricing_core):
        cfg = repricing_core.cfg
        if 'store' not in cfg or 'key' not in cfg['store'] or 'iv' not in cfg['store']:
            raise ValueError('Missing required configuration: "store>{key,iv}"')

        self.source_marketplace = repricing_core.source_marketplace
        self.api = repricing_core.bigcommerce_api
        self.sources = ['US', 'CA', 'UK', 'JP', 'DE', 'FR', 'ES', 'IT', 'MX']
        self.key = cfg['store']['key'].encode()
        self.iv = cfg['store']['iv'].encode()
        self.stats = repricing_core.stats

    def enumerate_inventory(self):
        for variant in self.api.CatalogVariants.iterall():
            self.stats.inc_value('inventory_total')

            marketplace, condition, asin = self.parse_sku(variant.sku)
            if marketplace != self.source_marketplace:
                self.stats.inc_value('inventory_invalid')
                logger.warning(
                    '[UnmatchMarketplace] source marketplace: %s, parsed marketplace: %s',
                    self.source_marketplace, marketplace)
                continue

            record = {
                'marketplace': marketplace,
                'condition': condition,
                'asin': asin,
                'variant': variant
            }
            self.stats.inc_value('inventory_processed')

            yield record

    def parse_sku(self, sku):
        decrypted_sku = decrypt(sku, self.key, self.iv)
        try:
            cond_num = int(decrypted_sku[0])
            condition = 'New' if cond_num % 2 != 0 else 'Any'
        except Exception as e:
            logger.exception(e)
            logger.warning(
                '[UnknownCondition] Could not parse condition. sku: %s, decrypted_sku: %s',
                sku, decrypted_sku)

            condition = 'New'

        try:
            marketplace_num = int(decrypted_sku[1])
            marketplace = self.sources[marketplace_num]
        except:
            logger.warning(
                '[UnknownMarketplace] Could not recognize marketplace. sku: %s, decrypted_sku: %s',
                sku, decrypted_sku)
            marketplace = 'US'

        asin = decrypted_sku.replace('-', '_').split('_').pop()

        return (marketplace, condition, asin)
