# -*- coding: utf-8 -*-

# Copyright © 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import os
import sys
import logging
import logging.handlers
from datetime import datetime

import click
import requests

from cmutils.config_loaders import YamlConfigLoader
from website_utils.google_shopping import GoogleShopping
from website_utils import logger


@click.command()
@click.option('-c', '--config_path', required=True, help='Configuration file path.')
def update_google_merchant_expiration(config_path):
    config_path = os.path.abspath(os.path.expanduser(config_path))
    if not os.path.isfile(config_path):
        logger.error('Could not find configuration file %s', config_path)
        sys.exit(1)

    cl = YamlConfigLoader(config_path)
    cfg = cl.load()

    work_dir = os.path.join(
        os.path.expanduser('~'), '.bigcommerce_reprice', cfg['store']['name'])
    if not os.path.isdir(work_dir):
        os.makedirs(work_dir)

    log_dir = os.path.join(work_dir, 'logs')
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)
    log_path = os.path.join(log_dir, 'google_merchant_expiration_update.log')
    level = logging.INFO
    max_bytes = 20 * 1024 ** 2
    fh = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=5)
    fh.setLevel(level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    if 'google_shopping' not in cfg['platforms']:
        logger.error('Missing required configuration "platforms>google_shopping"')
        sys.exit(1)

    google_shopping = GoogleShopping(cfg['platforms']['google_shopping'])
    result = google_shopping.update_expiration()
    payload = {
        'website': cfg['store']['name'],
        'merchant_id': result['merchant_id'],
        'total': result['total'],
        'expired': result['expired'],
        'expire_in_one_week': result['expire_in_one_week'],
        'succeed': result['succeed'],
        'failed': result['failed']
    }
    try:
        resp = requests.post(
            'http://reprice.olivesmall.com/google_expiration_update', json=payload,
            auth=(cfg['report']['user'], cfg['report']['password']))
        logger.info('[Reported] %s', str(resp.json()))
    except Exception as e:
        logger.exception(e)


if __name__ == '__main__':
    update_google_merchant_expiration()
